package com.fg.scripting.groovy.aop;

import com.fg.scripting.ProxyInstanceFactory;
import com.fg.scripting.ScriptingFactory;
import com.fg.scripting.ScriptingProxy;
import com.fg.scripting.exception.ClassInstantiationException;
import com.fg.scripting.groovy.monitoring.GroovyGarbageCollectorMonitor;
import org.aopalliance.intercept.MethodInvocation;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.aop.framework.Advised;
import org.springframework.aop.framework.AopContext;
import org.springframework.aop.support.DelegatingIntroductionInterceptor;
import org.springframework.aop.target.HotSwappableTargetSource;
import org.springframework.beans.CachedIntrospectionResults;

import java.beans.Introspector;
import java.lang.ref.WeakReference;
import java.lang.reflect.Method;

/**
 * Basic and probably unique GroovyProxy implementation.
 *
 * @author Jan Novotný, FG Forrest a.s. (c) 2007
 * @version $Id: GroovyProxyMixin.java,v 1.4 2009/11/15 20:05:12 u_novoj Exp $
 */
@SuppressWarnings({"serial", "NonSerializableFieldInSerializableClass", "AccessToStaticFieldLockedOnInstance"})
public class GroovyProxyMixin extends DelegatingIntroductionInterceptor implements ScriptingProxy {
	private static final Log log = LogFactory.getLog(GroovyProxyMixin.class);
	private static final ProxyInstanceFactory DEFAULT_INSTANCE_FACTORY = new DefaultProxyInstanceFactory();
	private final WeakReference<ScriptingFactory> groovyFactory;
	private final Object monitor = new Object();	
	private Object groovyInstance;
	private ProxyInstanceFactory instanceFactory;
	private long nextCheckTimestamp;

	public GroovyProxyMixin(ScriptingFactory groovyFactory, Object groovyInstance) {
		this.groovyFactory = new WeakReference<ScriptingFactory>(groovyFactory);
		this.nextCheckTimestamp = System.currentTimeMillis() + groovyFactory.getRefreshInterval();
		wrap(groovyInstance);
	}

	public void setInstanceInitializer(ProxyInstanceFactory factory) {
		this.instanceFactory = factory;
	}

	public ProxyInstanceFactory getInstanceInitializer() {
		if (instanceFactory == null) {
			return DEFAULT_INSTANCE_FACTORY;
		} else {
			return instanceFactory;
		}
	}

	public void wrap(Object scriptingInstance) {
		this.groovyInstance = scriptingInstance;
	}

	public Object unwrap() {
		return groovyInstance;
	}

	public boolean isUpToDate() {
		//check groovy factory is still living
		ScriptingFactory groovyFactory = this.groovyFactory.get();
		if (groovyFactory == null || groovyFactory.isClosed()) {
			this.groovyInstance = null;
			this.instanceFactory = null;
			throw new IllegalStateException("Groovy factory is already closed - this object reference is simply dead.");
		}
		//check inner groovy instance uptodate status
		long currentTimestamp = System.currentTimeMillis();
		if (currentTimestamp > nextCheckTimestamp) {
			nextCheckTimestamp = currentTimestamp + groovyFactory.getRefreshInterval();
			synchronized(monitor) {
				Class oldGroovyClass = groovyInstance.getClass();
				Class groovyClass = groovyFactory.loadClass(oldGroovyClass.getName());
				if(oldGroovyClass == groovyClass) {
					return true;
				} else {
					if(log.isDebugEnabled()) {
						log.debug("Class " + groovyClass.getName() + " changed - refreshing instance!");
					}
					GroovyGarbageCollectorMonitor.addGroovyClassToMonitoring(groovyClass);
					//underlying script has changed and Groovy classloader reloaded the class
					groovyInstance = getInstanceInitializer().createInstance(groovyClass);
					//change proxy target for the next call
					Advised advised = (Advised)AopContext.currentProxy();
					HotSwappableTargetSource targetSource = (HotSwappableTargetSource)advised.getTargetSource();
					targetSource.swap(groovyInstance);
					//clear introspection cache to be able to free classes
					CachedIntrospectionResults.clearClassLoader(groovyFactory.getScriptingClassLoader());
					Introspector.flushFromCaches(oldGroovyClass);
					return false;
				}
			}
		}
		return true;
	}

	@Override
	protected Object doProceed(MethodInvocation mi) throws Throwable {
		if(isUpToDate()) {
			return super.doProceed(mi);
		} else {
			//as we cannot change method invocation for this call, manually run method on newly created object
			//and immitate that the result goes from the existing MethodInvocation
			//possible exeptions of NoSuchMethodException and SecurityException propagate
			Class groovyClass = groovyInstance.getClass();
			Method originalMethod = mi.getMethod();
			Method newMethod = groovyClass.getMethod(originalMethod.getName(), originalMethod.getParameterTypes());
			return newMethod.invoke(groovyInstance, mi.getArguments());
		}
	}

	/**
	 * Default implementation.
	 */
	public static class DefaultProxyInstanceFactory implements ProxyInstanceFactory {

		public Object createInstance(Class clazz) throws ClassInstantiationException {
			try {
				return clazz.newInstance();
			}
			catch(Exception e) {
				String msg = "Cannot reinstantiate class: " + clazz.getName() + " (" + e.getLocalizedMessage() + ")";
				log.error(msg);
				throw new ClassInstantiationException(msg, e, clazz);
			}
		}

	}

}
