package com.fg.scripting.groovy.monitoring;

import groovy.lang.GroovyClassLoader;
import groovy.util.GroovyScriptEngine;

import java.util.Date;
import java.util.Map;
import java.util.WeakHashMap;

/**
 * This class is used to monitor whether groovy classes and classloaders are garbage collected or not.
 * Meant to be used for debugging purposes.
 *
 * @author Jan Novotný, FG Forrest a.s. (c) 2007
 * @version $Id: GroovyGarbageCollectorMonitor.java,v 1.3 2009/11/15 20:05:12 u_novoj Exp $
 */
@SuppressWarnings({"StaticMethodOnlyUsedInOneClass", "CallToSystemGC"})
public class GroovyGarbageCollectorMonitor {
	private static final Map<GroovyClassLoader, Date> groovyClassLoaders = new WeakHashMap<GroovyClassLoader, Date>();
	private static int groovyClassLoadersCount = 0;
	private static final Map<GroovyScriptEngine, Date> groovyScriptEngines = new WeakHashMap<GroovyScriptEngine, Date>();
	private static int groovyScriptEnginesCount = 0;
	private static final Map<Class, Date> groovyClasses = new WeakHashMap<Class, Date>();
	private static int groovyClassesCount = 0;

	private GroovyGarbageCollectorMonitor() {}

	/**
	 * Adds GroovyClassloader to monitoring evidence.
	 * @param classLoader
	 */
	public static void addGroovyClassLoaderToMonitoring(GroovyClassLoader classLoader) {
		synchronized(groovyClassLoaders) {
			groovyClassLoaders.put(classLoader, new Date());
			groovyClassLoadersCount++;
		}
	}

	/**
	 * Adds GroovyScriptingEngine to monitoring evidence.
	 * @param groovyScriptEngine
	 */
	public static void addGroovyScriptEngineToMonitoring(GroovyScriptEngine groovyScriptEngine) {
		synchronized(groovyScriptEngines) {
			groovyScriptEngines.put(groovyScriptEngine, new Date());
			groovyScriptEnginesCount++;
		}
	}

	/**
	 * Adds Groovy class to monitoring evidence.
	 * @param clazz
	 */
	public static void addGroovyClassToMonitoring(Class clazz) {
		synchronized(groovyClasses) {
			groovyClasses.put(clazz, new Date());
			groovyClassesCount++;
		}
	}

	/**
	 * Returns count of garbage collected GroovyClassLoaders.
	 * @return
	 */
	public static int getGarbageCollectedGroovyClassLoaderCount() {
		System.gc();
		synchronized(groovyClassLoaders) {
			return groovyClassLoadersCount - groovyClassLoaders.size();
		}
	}

	/**
	 * Returns count of living (not yet garbage collected) GroovyClassLoaders.
	 * @return
	 */
	public static int getLivingGroovyClassLoaderCount() {
		System.gc();
		synchronized(groovyClassLoaders) {
			return groovyClassLoaders.size();
		}
	}

	/**
	 * Returns count of garbage collected CroovyScriptEngines.
	 * @return
	 */
	public static int getGarbageCollectedGroovyScriptEnginesCount() {
		System.gc();
		synchronized(groovyScriptEngines) {
			return groovyScriptEnginesCount - groovyScriptEngines.size();
		}
	}

	/**
	 * Returns count of living (not yet garbage collected) CroovyScriptEngines.
	 * @return
	 */
	public static int getLivingGroovyScriptEnginesCount() {
		System.gc();
		synchronized(groovyScriptEngines) {
			return groovyScriptEngines.size();
		}
	}

	/**
	 * Returns count of garbage collected Groovy classes.
	 * @return
	 */
	public static int getGarbageCollectedClassCount() {
		System.gc();
		synchronized(groovyClasses) {
			return groovyClassesCount - groovyClasses.size();
		}
	}

	/**
	 * Returns count of living (not yet garbage collected) Groovy classes.
	 * @return
	 */
	public static int getLivingGroovyClassCount() {
		System.gc();
		synchronized(groovyClasses) {
			return groovyClasses.size();
		}
	}

}
