package com.fg.scripting.web;

import com.fg.scripting.*;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import java.io.IOException;

import static com.fg.scripting.web.ScriptingDelegationUtils.*;

/**
 * This proxy delegates Servlet calls to scripted instance (fe. Groovy).
 *
 * @author Jan Novotný, FG Forrest a.s. (c) 2007
 * @version $Id: ScriptingDelegatingFilterProxy.java,v 1.6 2010/01/14 12:30:27 u_novoj Exp $
 */
public class ScriptingDelegatingFilterProxy implements Filter {
	private static final Log log = LogFactory.getLog(ScriptingDelegatingServletProxy.class);
	protected FilterConfig filterConfig;
	protected Filter delegate;
	private final Object delegateMonitor = new Object();

	public void init(FilterConfig filterConfig) throws ServletException {
		this.filterConfig = filterConfig;
		initDelegate();
	}

	/**
	 * Performs delegate initialization.
	 * @throws ServletException
	 */
	protected void initDelegate() throws ServletException {
		String targetClass = filterConfig.getInitParameter("targetClass");
		try {
			synchronized(delegateMonitor) {				
				if (isEmpty(targetClass)) {
					String msg = "Cannot instantiate scripting delegate - no targetClass init parameter defined for the filter!";
					log.error(msg);
					return;
				}
				ScriptingFactory fct = getScriptingFactory(filterConfig);
				if (fct == null) {
					String msg = "Cannot instantiate scripting delegate (" + targetClass + ") - no scripting factory available!";
					log.error(msg);
					return;
				}
				Object instance = fct.createInstance(targetClass);
				if (!(instance instanceof Filter)) {
					String msg = "Cannot use scripting delegate (" + targetClass + ") - it doesn't represetn Filter class!";
					log.error(msg);
					return;
				}
				this.delegate = (Filter)instance;
				this.delegate.init(filterConfig);
			}
		} catch (Throwable ex) {
			String msg = "Cannot instantiate scripting delegate (" + targetClass + "): " + ex.getLocalizedMessage();
			log.error(msg);
		}
	}

	/**
	 * Initialize groovy scripting factory from the init parameters.
	 * @return
	 */
	protected ScriptingFactory getScriptingFactory(FilterConfig filterConfig) {
		String factoryClassName = filterConfig.getInitParameter("factory");
		String scriptDirectory = filterConfig.getInitParameter("scriptDir");
		String encoding = filterConfig.getInitParameter("encoding");
		if (encoding == null) {
			encoding = "UTF-8";
		}
		return getNewScriptingFactory(factoryClassName, scriptDirectory, filterConfig.getServletContext(), encoding);
	}

	public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) throws IOException, ServletException {
		//try to lazily init delegate
		if (this.delegate == null) {
			initDelegate();
		}
		if (this.delegate != null) {
			this.delegate.doFilter(servletRequest, servletResponse, filterChain);
		} else {
			if(log.isDebugEnabled()) {
				log.debug("No scripting filter delegate available, executing other filters in filter chain.");
			}
			filterChain.doFilter(servletRequest, servletResponse);
		}
	}

	public void destroy() {
		if (this.delegate != null) {
			this.delegate.destroy();
		}
		this.delegate = null;
	}
}
