package com.fg.scripting.web;

import com.fg.scripting.*;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

import static com.fg.scripting.web.ScriptingDelegationUtils.*;

/**
 * This proxy delegates Servlet calls to scripted instance (fe. Groovy).
 *
 * @author Jan Novotný, FG Forrest a.s. (c) 2007
 * @version $Id: ScriptingDelegatingServletProxy.java,v 1.5 2010/01/14 12:30:27 u_novoj Exp $
 */
public class ScriptingDelegatingServletProxy extends HttpServlet {
	private static final long serialVersionUID = -1978529787551019405L;
	private static final Log log = LogFactory.getLog(ScriptingDelegatingServletProxy.class);
	protected HttpServlet delegate;
	@SuppressWarnings({"NonSerializableFieldInSerializableClass"})
	private final Object delegateMonitor = new Object();

	@Override
	public void init() throws ServletException {
		initDelegate();
	}

	/**
	 * Performs delegate initialization.
	 * @throws ServletException
	 */
	protected void initDelegate() throws ServletException {
		String targetClass = getInitParameter("targetClass");
		try {
			synchronized(delegateMonitor) {
				if (isEmpty(targetClass)) {
					String msg = "Cannot instantiate scripting delegate - no targetClass init parameter defined for the servlet!";
					log.error(msg);
					return;
				}
				ScriptingFactory fct = getScriptingFactory();
				if (fct == null) {
					String msg = "Cannot instantiate scripting delegate (" + targetClass + ") - no scripting factory available!";
					log.error(msg);
					return;
				}
				Object instance = fct.createInstance(targetClass);
				if (!(instance instanceof HttpServlet)) {
					String msg = "Cannot use scripting delegate (" + targetClass + ") - it doesn't represent HttpServlet class!";
					log.error(msg);
					return;
				}
				this.delegate = (HttpServlet)instance;
				this.delegate.init(getServletConfig());
			}
		} catch (Throwable ex) {
			String msg = "Cannot instantiate scripting delegate (" + targetClass + "): " + ex.getLocalizedMessage();
			log.error(msg);
		}
	}

	/**
	 * Initialize groovy scripting factory from the init parameters.
	 * @return
	 */
	protected ScriptingFactory getScriptingFactory() {
		String factoryClassName = getInitParameter("factory");
		String scriptDirectory = getInitParameter("scriptDir");
		String encoding = getInitParameter("encoding");
		if (encoding == null) {
			encoding = "UTF-8";
		}
		return getNewScriptingFactory(factoryClassName, scriptDirectory, getServletContext(), encoding);
	}

	@Override
	public void service(ServletRequest req, ServletResponse res) throws ServletException, IOException {
		//try to lazily init delegate
		if (this.delegate == null) {
			initDelegate();
		}
		if (this.delegate != null) {
			this.delegate.service(req, res);
		}
	}

	@Override
	protected void service(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		//try to lazily init delegate
		if (this.delegate == null) {
			initDelegate();
		}
		if (this.delegate != null) {
			this.delegate.service(req, resp);
		} else {
			resp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
		}
	}

	@Override
	public void destroy() {
		super.destroy();
		if (this.delegate != null) {
			this.delegate.destroy();
		}
		this.delegate = null;
	}

}
